/*
 *  Open.Theremin.UNO control software for Arduino.UNO
 *  Version 1.2.1
 *  Copyright (C) 2010-2013 by Urs Gaudenz
 *  adding the pitch knob and waveform selector by yasuski
 *
 *  Open.Theremin.UNO control software is free software: you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Open.Theremin.UNO control software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  the Open.Theremin.UNO control software.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <avr/io.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <avr/delay.h>



#include "mcpDac.h"
#include "theremin_sintable.c"
#include "theremin_sintable2.c"    //adding the extra waveforms
#include "theremin_sintable3.c"
#include "theremin_sintable4.c"
#include "theremin_sintable5.c"
#include "theremin_sintable6.c"
#include "theremin_sintable7.c"
#include "theremin_sintable8.c"
#include "theremin_sintable9.c"
#include "theremin_sintable10.c"
#include "theremin_sintable11.c"
#include "theremin_sintable12.c"


#define F_CPU 16000000UL
#define BAUD 9600

#define ARDUINO_LEONARDO

#if defined ARDUINO_LEONARDO
#define USBSERIAL Serial
#elif defined ROBOT_MOTOR
#define USBSERIAL Serial1
#elif defined DUE
#define USBSERIAL SerialUSB
#endif defined MAPLE
#define USBSERIAL SerialUSB

#define val_MIN 12                  //map value for the volume pots
#define val_MAX 1
#define def_MIN 0
#define def_MAX 1023

#define val_MIN2 1023
#define val_MAX2 2
#define def_MIN2 1
#define def_MAX2 1023


#define PA0
#define PA1

#include <util/setbaud.h>

/* General variables */
int32_t pitch_init = 0; 			// Initialization value of pitch
int32_t vol_init = 0;				// Initialization value of volume
int32_t pitch_v,pitch_l;			// Last value of pitch (for filtering)
int32_t vol_v,vol_l;				// Last value of volume (for filtering)
uint16_t pitch = 0;				// Pitch value
uint16_t pitch_counter = 0; 			// Pitch counter
uint16_t pitch_counter_l = 0; 			// Last value of pitch counter
uint16_t vol = 0;  				// Volume value
uint16_t vol_counter_l = 0;			// Last value of volume counter

uint16_t pot1 = 0;                              // values of the volume pots
uint16_t pot2 = 0;
uint16_t pot3 = 0;
uint16_t addr1 = 1;
uint16_t addr2 = 1;

uint8_t i;
uint8_t state = 0;				// State in the calibration state machine

uint8_t mode = 4; 				// Calibration mode

uint8_t waveTable = 12;                          // address of the waveTables

/* volatile varables  - used in the ISR Routine*/ 
volatile uint8_t vol8;				// Volume byte
volatile bool flag_vol = 0;			// Volume read flag
volatile uint16_t vol_counter = 0; 		// Volume counter
volatile uint16_t pointer = 0;			// Table pointer
volatile uint16_t pointer2 = 0;			// Table pointer
volatile uint16_t add_val,add_val2 = 0;	        // Table pointer increment
volatile uint16_t timer = 0; 			// Timer value

/* 16 bit by 8 bit multiplication */
static inline uint32_t mul_16_8(uint16_t a, uint8_t b)	
{
       uint32_t product;
       asm (
               "mul %A1, %2\n\t"
               "movw %A0, r0\n\t"
               "clr %C0\n\t"
               "clr %D0\n\t"
               "mul %B1, %2\n\t"
               "add %B0, r0\n\t"
               "adc %C0, r1\n\t"
               "clr r1"
               : "=&r" (product)
               : "r" (a), "r" (b));
       return product;
}
 
/* initialization routine for uart (serial) communication */
void uart_init(void) {				
    UBRR0H = UBRRH_VALUE;
    UBRR0L = UBRRL_VALUE;

#if USE_2X
    UCSR0A |= _BV(U2X0);
#else
    UCSR0A &= ~(_BV(U2X0));
#endif

    UCSR0C = _BV(UCSZ01) | _BV(UCSZ00); 	// 8-bit data
    UCSR0B = _BV(RXEN0) | _BV(TXEN0);   	// Enable RX and TX
}

/* Routine for sending character on uart */
void uart_putchar(char c) {			
    loop_until_bit_is_set(UCSR0A, UDRE0); 	// Wait until data register empty. 
    UDR0 = c;
}

/* Externaly generated 31250 Hz Interrupt for WAVE generator (32us) */
ISR (INT1_vect)                         
{
  					// Interrupt takes up a total of max 25 us
EIMSK &= ~ (1<<INT1); 			// Disable External Interrupt INT1 to avoid recursive interrupts
sei(); 					// Enable Interrupts to allow counter 1 interrupts
   
    int16_t temp_val; 			// temporary variable 1
    int16_t temp_val2; 			// temporary variable 1

    uint32_t temp2_val; 		// temporary variable 2

switch (addr1)					// set wave tables for reading

	{
		case 1:  temp_val = (signed int)pgm_read_word_near (sine_table + ((unsigned int)(pointer>>6) & 0x3ff));break;  // read Table1 
		case 2:  temp_val = (signed int)pgm_read_word_near (sine_table2 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table2
		case 3:  temp_val = (signed int)pgm_read_word_near (sine_table3 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table3
		case 4:  temp_val = (signed int)pgm_read_word_near (sine_table4 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table4
                case 5:  temp_val = (signed int)pgm_read_word_near (sine_table5 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table5 
		case 6:  temp_val = (signed int)pgm_read_word_near (sine_table6 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table6
		case 7:  temp_val = (signed int)pgm_read_word_near (sine_table7 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table7
		case 8:  temp_val = (signed int)pgm_read_word_near (sine_table8 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table8
                case 9:  temp_val = (signed int)pgm_read_word_near (sine_table9 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table9 
		case 10:  temp_val = (signed int)pgm_read_word_near (sine_table10 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table10
		case 11:  temp_val = (signed int)pgm_read_word_near (sine_table11 + ((unsigned int)(pointer>>6) & 0x3ff));break;   // read Table11
		case 12:  temp_val = (signed int)pgm_read_word_near (sine_table12 + ((unsigned int)(pointer>>6) & 0x3ff));break;  // read Table12 
                };


//  temp_val = (signed int)pgm_read_word_near (sine_table + ((unsigned int)(pointer>>6) & 0x3ff));  //Read next wave table value (3.0us)
  
//    PORTD |= (1<<PORTD5);


 if (temp_val>0){                   	// multiply 16 bit wave number by 8 bit volume value (11.2us / 5.4us)
   temp2_val=mul_16_8(temp_val,vol8);
   temp2_val=temp2_val>>9; 
   temp2_val=temp2_val+1748;  
 } else
 { 
   temp2_val=mul_16_8(-temp_val,vol8);
   temp2_val=temp2_val>>9; 
   temp2_val=1748-temp2_val;
 }

/* 
  if (temp_val2>0){                   	// multiply 16 bit wave number by 8 bit volume value (11.2us / 5.4us)
   temp2_val=mul_16_8(temp_val2,vol8);
   temp2_val=temp2_val>>9; 
   temp2_val=temp2_val+1748;  
 } else
 { 
   temp2_val=mul_16_8(-temp_val2,vol8);
   temp2_val=temp2_val>>9; 
   temp2_val=1748-temp2_val;
 }

  if (temp_val>0){                   	// multiply 16 bit wave number by 8 bit volume value (11.2us / 5.4us)
   temp2_val=temp_val;
   temp2_val=temp2_val>>1; 
   temp2_val=temp2_val+1748;  
 } else
 { 
   temp2_val=-temp_val;
   temp2_val=temp2_val>>1; 
   temp2_val=1748-temp2_val;
 }
 */

//  PORTD &= ~(1<<PORTD5);

  mcpDacSend(temp2_val);  		//Send result to Digital to Analogue Converter (audio out) (9.6 us)

  pointer = pointer + add_val;		// increment table pointer (ca. 2us)
  pointer2 = pointer2 + add_val2;
  
  
  timer++;				// update 32us timer

cli();  				// Turn of interrupts
EIMSK |= (1<<INT1); 			// Re-Enable External Interrupt INT1
   
}

/* VOLUME read - interrupt service routine for capturing volume counter value */
ISR (INT0_vect) 			
{	
vol_counter = TCNT1;
flag_vol=true;                          // Set new volume value flag
};


void ticktimer (int ticks)        	//Wait for ticks * 32 us
{
  timer=0;while(timer<ticks);
};

/* Initialize pitch and volume values for calibration */
void InitValues(void)			
{
// Set initial pitch value



	pitch_counter_l=ICR1;                    	// Store actual Timer 1 counter value
      	TIFR1 = (1<<ICF1);				// Clear capture flag
      	timer=0;
        while (!(TIFR1&(1<<ICF1))&&(timer<312));	// Waite for new value (with exit after 10 ms)
	pitch_init=(ICR1-pitch_counter_l);              // Counter change since last interrupt = init pitch value
				
// Set initial volume value   
	vol_counter_l=vol_counter;   			// Store actual counter value
        flag_vol=false;					// Clear volume flag
        timer=0;
        while(!flag_vol&&(timer<312));			// Waite for new value (with exit after 10 ms)
        vol_init=(vol_counter-vol_counter_l);           // Counter change since last interrupt = init volume value

}

/* General Setup Routine */

int addrA = 1;
int addrB = 1;
int sw1 = 1;
int sw2 = 1;
int sw1m;
int sw2m;


void setup() {
  

pinMode(9, INPUT_PULLUP);
pinMode(5, INPUT_PULLUP);
pinMode(4, INPUT_PULLUP);
// pinMode(sw3, INPUT);
// pinMode(sw4, INPUT);

//PORTD = (1<<PORTD0);
//PORTD = (1<<PORTD1);
//PORTD = (1<<PORTD4);            // Set Pull-Up on Button Pin D4 (BUT)
//PORTB = (1<<PORTB1);            
//PORTD = (1<<PORTD5);  
//Serial.begin(9600);
//Serial.println("START");

/* Setup Timer 1, 16 bit timer used to measure pitch and volume frequency */

TCCR1A = 0;                     // Set Timer 1 to Normal port operation (Arduino does activate something here ?)
TCCR1B = (1<<ICES1)|(1<<CS10);  // Input Capture Positiv Edge Select, Run without prescaling (16 Mhz)

/* Setup Inerrupts for Wave Generator and Volume read */
EICRA = (1<<ISC00)|(1<<ISC01)|(1<<ISC11)|(1<<ISC10) ; // The rising edges of INT0 and INT1 generate an interrupt request.
EIMSK = (1<<INT0)|(1<<INT1);    // Enable External Interrupt INT0 and INT1

sei (); 			// Enable Interrupts

mcpDacInit();			// Initialize Digital Analog Converter (DAC)


// Init PWM Generator for CV out (if used)
TCCR0A = (1<<COM0A1)|(1<<WGM01)|(1<<WGM00); //Set PWM on OC0A
TCCR0B = (1<<CS00); // Set Clock with No prescaling on Timer 0
TIMSK0=0; // No interrups
DDRD |= (1<<PORTD6);       // Set Pin D6 as output


//uart_init(); // Start Serial Communication (if used)


vol8=255;				// Set volume to max
//        add_val2 =600;
	add_val = 500;			// Play welcome sounds
	ticktimer(15000);
	add_val = 1000;
//        add_val2 =1200;
	ticktimer(15000);
	add_val = 2000;
//        add_val2 =2500;
	ticktimer(30000);

        InitValues();    		// Capture initial calibration values
}



/* Main Loop */
void loop() {

mloop: 							// Main loop avoiding the GCC "optimization"	

  //Handle input




int pota = analogRead(5);                               // scanning the values of the volume pots
         pota = map(pota, def_MIN2, def_MAX2, val_MIN2, val_MAX2);
         pota = (pota + (pot1/4))/2;
         pot1 = pota * 4 - pota;

//int potb = analogRead(3);
//         potb = map(potb, def_MIN, def_MAX, val_MIN, val_MAX);
//         pot2 = potb;
//         Serial.println(pot2); 

int potc = analogRead(4);                               // scanning the values of the volume pots
         potc = map(pota, def_MIN2, def_MAX2, val_MIN2, val_MAX2);
         potc = (potc + (pot3/4))/2;
         pot3 = potc * 4 - potc;

if ((state==0)&&((PIND&(1<<PORTD4))==0))		// Check if key released
	{state=1;
	timer=0;}

if ((state==1)&&((PIND&(1<<PORTD4))!=0))		// If key pressed
	{if (timer > 1500)
	{
   	ticktimer(100000);                               // away from sensing area
  	vol8=255;					// Play calibration sounds
	add_val = 700;
	ticktimer(15000);
	vol8=0;
	ticktimer(3000);
	vol8=255;
	ticktimer(15000);
	vol8=0;
	ticktimer(3000);
	vol8=255;				
	ticktimer(15000);
	vol8=0;
	ticktimer(3000);
	add_val = 2000;
	vol8=255;
	ticktimer(3000);

        InitValues();					// Capture calibration Values
	state=0;
	mode=4;} else {state=0;}
        };
	
if ((state==1)&&(timer>20000))				// If key pressed for >64 ms switch calibration modes
	{
	state=0;
	mode++;

	if (mode>4){mode=1;};
	for (i=0;i<mode;i++)
	{
	vol8=255;
	add_val = 3000;	
	ticktimer(1500);
	vol8=0;
	ticktimer(1500);
	vol8=255;
	add_val = 0;	
	};

	while((PIND&(1<<PORTD4))==0)
	{};
	};



OCR0A=pitch&0xff;					// Set CV value (if used)

if (timer>3125){
 timer=0;

 uart_putchar(pitch&0xff);				// Send char on serial (if used)
 uart_putchar((pitch>>8)&0xff);

}


/* New PITCH value */

if (TIFR1&(1<<ICF1)){					// If capture event

        pitch_counter=ICR1;				// Get Timer-Counter 1 value
	pitch=(pitch_counter-pitch_counter_l);		// Counter change since last interrupt -> pitch value
	pitch_counter_l=pitch_counter;			// Set actual value as new last value

	//if ((pitch>19000)&(pitch<21000)) {PORTA |= (1<<PA0);} else {PORTA &= ~(1<<PA0);}	// LED on if value in range

	pitch_v=pitch;					// Averaging pitch values
	pitch_v=pitch_l + ((pitch_v-pitch_l)>>2);	 
	pitch_l=pitch_v + pot1;	                        // adding the values from the pitch knob

	switch (mode)					// set wave frequency for each mode
	{
		case 1:add_val=33554432/pitch_v;break; 		 // pitch calibration mode
		case 2:break;					 // volume calibration mode
		case 3:add_val=(pitch_init-pitch_v)/2+200;break; // mode without volume
		case 4:add_val=(pitch_init-pitch_v)/2+200;break; // normal operation
	};

    TIFR1 = (1<<ICF1);                  		// Clear pitch capture flag
}

/* New VOLUME value */

if (flag_vol){

        vol=(vol_counter-vol_counter_l);		// Counter change since last interrupt 
	vol_counter_l=vol_counter;			// Set actual value as new last value

	vol_v=vol;					// Averaging volume values
	vol_v=vol_l+((vol_v-vol_l)>>2);
	vol_l=vol_v + pot3;                             // adding the values from the volume knob

	//if ((vol_v>15000)&(vol_v>17000)) {PORTA |= (1<<PA1);} else {PORTA &= ~(1<<PA1);} // LED on if value in range

	switch (mode)					// set volume for each mode

	{
		case 1:vol_v=4095;break;			// pitch calibration mode 
		case 2:add_val=33554432/vol_v;vol_v=4095;break;	// volume calibration mode
		case 3:vol_v=4095;break;			// mode without volume
		case 4:vol_v=4095-(vol_init-vol_v);break;	// normal operation
	};

	

	if (vol_v>4095) {vol8=255;} else			// Limit and set volume value
	if (vol_v<0) {vol8=0;} else {
	vol8=vol_v>>4;}

	flag_vol=false;						// Clear volume flag

}

  sw1 = digitalRead(9);
  
  if((sw1 < 1) & (sw1m == 1))
  {
    addrA = addrA + 1;
    //Reset count if over max mode number
    if(addrA > 12)
    {
      addrA = 1;

    }
    
  }
 sw1m = digitalRead(9);
 sw2 = digitalRead(5);  

 if((sw2 < 1) & (sw2m == 1))
  {
    addrA = addrA - 1;
    //Reset count if over max mode number
    if(addrA < 1)
    {
      addrA = 12;

    }
     
  }
sw2m = digitalRead(5);
 addr1 = addrA;

goto mloop; 							// End of main loop	
}





